/*-----------------------------------------------------------------------------
 * Copyright (c) 2013 - 2019 Arm Limited (or its affiliates). All
 * rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   1.Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   2.Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   3.Neither the name of Arm nor the names of its contributors may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * Name:    Joystick_MCB1800.c
 * Purpose: Joystick interface for MCB1800 evaluation board
 * Rev.:    1.1
   ---------------------------------------------------------------------------*/

#include "LPC18xx.h"
#include "GPIO_LPC18xx.h"
#include "SCU_LPC18xx.h"
#include "Board_Joystick.h"

#define JOYSTICK_COUNT                 (5U)

/* Joystick pins:
   - center: PC_9  = GPIO6[8]
   - up:     PC_11 = GPIO6[10]
   - down:   PC_12 = GPIO6[11]
   - left:   PC_13 = GPIO6[12]
   - right:  PC_14 = GPIO6[13] */

/* Joystick GPIO definitions */
static const GPIO_ID JOYSTICK_GPIO[] = {
  { 6U,  8U },
  { 6U, 10U },
  { 6U, 11U },
  { 6U, 12U },
  { 6U, 13U }
};

/* Joystick pin definitions */
static const PIN_ID JOYSTICK_PIN[] = {
  { 12U,  9U, {0U,0U}, (SCU_CFG_MODE_FUNC4 | SCU_SFS_EZI)},
  { 12U, 11U, {0U,0U}, (SCU_CFG_MODE_FUNC4 | SCU_SFS_EZI)},
  { 12u, 12U, {0U,0U}, (SCU_CFG_MODE_FUNC4 | SCU_SFS_EZI)},
  { 12U, 13U, {0U,0U}, (SCU_CFG_MODE_FUNC4 | SCU_SFS_EZI)},
  { 12U, 14U, {0U,0U}, (SCU_CFG_MODE_FUNC4 | SCU_SFS_EZI)}
};


/**
  \fn          int32_t Joystick_Initialize (void)
  \brief       Initialize joystick
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Joystick_Initialize (void) {
  uint32_t n;

  /* Enable GPIO clock */
  GPIO_PortClock     (1);

  /* Configure pins */
  for (n = 0U; n < JOYSTICK_COUNT; n++) {
    SCU_PinConfigure (JOYSTICK_PIN[n].port,  JOYSTICK_PIN[n].num,  JOYSTICK_PIN[n].config_val);
    GPIO_SetDir      (JOYSTICK_GPIO[n].port, JOYSTICK_GPIO[n].num, GPIO_DIR_INPUT);
  }
  return 0;
}

/**
  \fn          int32_t Joystick_Uninitialize (void)
  \brief       De-initialize joystick
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Joystick_Uninitialize (void) {
  uint32_t n;

  /* Unconfigure pins */
  for (n = 0U; n < JOYSTICK_COUNT; n++) {
    SCU_PinConfigure (JOYSTICK_PIN[n].port,  JOYSTICK_PIN[n].num,  0);
  }
  return 0;
}

/**
  \fn          uint32_t Joystick_GetState (void)
  \brief       Get joystick state
  \returns     Joystick state
*/
uint32_t Joystick_GetState (void) {
  uint32_t val;

  val  = 0;
  if (!(GPIO_PinRead (JOYSTICK_GPIO[0].port, JOYSTICK_GPIO[0].num))) val |= JOYSTICK_CENTER;
  if (!(GPIO_PinRead (JOYSTICK_GPIO[1].port, JOYSTICK_GPIO[1].num))) val |= JOYSTICK_UP;
  if (!(GPIO_PinRead (JOYSTICK_GPIO[2].port, JOYSTICK_GPIO[2].num))) val |= JOYSTICK_DOWN;
  if (!(GPIO_PinRead (JOYSTICK_GPIO[3].port, JOYSTICK_GPIO[3].num))) val |= JOYSTICK_LEFT;
  if (!(GPIO_PinRead (JOYSTICK_GPIO[4].port, JOYSTICK_GPIO[4].num))) val |= JOYSTICK_RIGHT;

  return val;
}
