/*-----------------------------------------------------------------------------
 * Copyright (c) 2004-2019 Arm Limited (or its affiliates). All
 * rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   1.Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   2.Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   3.Neither the name of Arm nor the names of its contributors may be used
 *     to endorse or promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * Name:    Demo.c
 * Purpose: Demo example for MCB1800
 * Note(s):
 *----------------------------------------------------------------------------*/

#include "main.h"
#include <stdio.h>
#include "Bulb_16bpp.h"
#include "Button_16bpp.h"
#include "Board_LED.h"
#include "Board_Buttons.h"
#include "Board_Joystick.h"
#include "Board_ADC.h"
#include "GLCD_Fonts.h"
#include "GLCD_Config.h"
#include "Board_Touch.h"
#include "Board_Thermometer.h"
#include "Board_EEPROM.h"

/* Main stack size must be multiple of 8 Bytes */
#define APP_MAIN_STK_SZ (1024U)
static uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

/* Button position definitions */
#define BTN_X        (9U)
#define BTN_Y        (190U)

/* Bulb position definition */
#define BLP_X        (209U)
#define BLP_Y        (190U)

/* Temperature graph definitions */
#define GRAPH_SAMPLES 240U              /* Number of samples to draw          */
#define GRAPH_Y_ORIG  220U              /* Coordinate system Y origin (pixel) */
#define GRAPH_X_ORIG   32U              /* Coordinate system X origin (pixel) */

#define AXIS_Y_LEN    120U              /* Length of the Y axis               */
#define AXIS_X_LEN    250U              /* Length of the X axis               */


/* Buttons bit masks */
#define BTN_P4_0     (1U << 0)
#define BTN_P4_3     (1U << 1)
#define BTN_P4_4     (1U << 2)
#define BTN_WAKEUP   (1U << 3)
 
/* Mode definitions */
enum {
  M_INIT = 0,
  M_SCR1,
  M_SCR2,
};

/* Periodic timer definition */
static void Timer_Callback (void *arg);
static osTimerId_t TimerId;

/* EEPROM Logger definitions */
typedef struct {
  uint16_t MemSz;
  uint16_t WrAddr;
  uint16_t WrCount;
} EE_LOG;

static EE_LOG          Log;
 
/* Mode switch timeout variable */
static uint32_t        BtnTick, Measure;

/* GLCD string buffer */
#define STRINGBUF_LEN 21U

static char            StringBuf[STRINGBUF_LEN];

/* Static variables */
static uint16_t Sample[GRAPH_SAMPLES];
static uint32_t LEDOn, LEDOff;

/*-----------------------------------------------------------------------------
  Periodic timer callback
 *----------------------------------------------------------------------------*/
static void Timer_Callback (void *arg) {
  static uint8_t TickLed, TickBtn, TickTh;

  (void) arg;

  switch (TickLed++) {
    case  1: LEDOn   = 1; break;
    case  6: LEDOff  = 1; break;
    case 10: TickLed = 0; break;
  }

  if (BtnTick == 0) {
    TickBtn++;
    if (TickBtn == 50) {
      BtnTick = 1;
      TickBtn = 0;
    }
  }
  else TickBtn = 0;
  
  TickTh++;
  if (TickTh > 200) {
    TickTh  = 0;
    Measure = 1;
  }
}


/*-----------------------------------------------------------------------------
  Read temperature from the EEPROM log
 *----------------------------------------------------------------------------*/
static uint32_t LOG_Read (uint16_t addr, uint16_t *temp, uint16_t *count) {
  uint8_t  dat[4];

  if (EEPROM_ReadData (addr, dat, 4U) == 0) {
    if ((dat[1] & 0xF0U) == 0U) {
      *temp  = (((uint32_t)dat[1] << 8) | dat[0]) & 0xFFFFU;
      *count = (((uint32_t)dat[3] << 8) | dat[2]) & 0xFFFFU;
      return (0U);
    }
  }
  return (1U);
}


/*-----------------------------------------------------------------------------
  Write temperature to the EEPROM log
 *----------------------------------------------------------------------------*/
static uint32_t LOG_Write (uint16_t temp) {
  uint32_t err;
  uint8_t  dat[4];

  dat[0] = (uint8_t)(temp        &  0xFFU);
  dat[1] = (uint8_t)(temp        >>     8);
  dat[2] = (uint8_t)(Log.WrCount &  0xFFU);
  dat[3] = (uint8_t)(Log.WrCount >>     8);

  err = (uint32_t) EEPROM_WriteData (Log.WrAddr, dat, 4U);

  Log.WrAddr += 4U;
  if (Log.WrAddr >= Log.MemSz) {
    Log.WrAddr = 0U;
  }

  Log.WrCount++;
  if (Log.WrCount == 0xFFFF) {
    Log.WrCount = 0U;                   /* Max count is 0xFFFE                */
  }
  return (err);
}


/*-----------------------------------------------------------------------------
  Erase EEPROM log data
 *----------------------------------------------------------------------------*/
static void LOG_Clear (void) {
  uint32_t adr;
  uint16_t count;
  uint8_t  buf[16];

  for (count = 0U; count < 16U; count++) buf[count] = 0xFFU;
  for (adr = 0U; (adr + 16U) <= Log.MemSz; adr += 16U) {
    EEPROM_WriteData (adr, buf, 16U);
  }
  if (Log.MemSz > adr) {
    EEPROM_WriteData (adr, buf, Log.MemSz - adr);
  }
}


/*-----------------------------------------------------------------------------
  Init EEPROM Logger
 *----------------------------------------------------------------------------*/
static uint32_t LOG_Init (void) {
  uint16_t adr, cc, nc;
  uint16_t temp, count;
  uint8_t  cnt[2];

  Log.MemSz   = EEPROM_GetSize() & 0xFFFFU;
  Log.WrCount = 0U;

  /* Erase EEPROM if temp data is garbage or if P4_0 button is pressed on init*/
  if ((LOG_Read (0U, &temp, &count) == 1U) || (Buttons_GetState() & BTN_P4_0)) {
    LOG_Clear();
  }

  /* Find last entry */
  for (adr = 0U; adr < Log.MemSz; adr += 8U) {
    Log.WrAddr  = adr;

    EEPROM_ReadData (adr+2U, cnt, 2U);
    cc = (((uint32_t)cnt[1] << 8) | cnt[0]) & 0xFFFFU;

    EEPROM_ReadData (adr+6U, cnt, 2U);
    nc = (((uint32_t)cnt[1] << 8) | cnt[0]) & 0xFFFFU;

    if (cc == 0xFFFFU) {                /* CC is empty entry, W = CC          */
      break;
    }    
    if (nc == 0xFFFFU) {                /* NC is empty entry, W = NC          */
      Log.WrAddr  += 4U;
      Log.WrCount += 1U;
      break;
    }
    if (cc > nc) {                      /* CC is last valid entry, W = NC     */
      if (!(cc == 0xFFFEU && nc == 0x0000U)) {
        /* This is not counter overlap, W = NC */
        Log.WrAddr  += 4U;
        Log.WrCount += 1U;
        break;
      }
    }
    Log.WrCount += 2U;
  }
  return (0U);
}


/*-----------------------------------------------------------------------------
  Convert temperature to pixels along Y coordinate
 *----------------------------------------------------------------------------*/
static uint32_t TempToY (uint16_t temp) {
  uint32_t t, px;

  t = (temp / 10U);

  if (t >= 40U) return (GRAPH_Y_ORIG - AXIS_Y_LEN);
  if (t <= 20U) return (GRAPH_Y_ORIG);

  px = (6U * (t-20U));
  if (temp % 10U) px += 3U;

  return (GRAPH_Y_ORIG - px);
}


/*-----------------------------------------------------------------------------
  Connect two consecutive samples with a straight line
 *----------------------------------------------------------------------------*/
static void ConnectSample (uint32_t x1, uint32_t y1, uint32_t y2) {
  int32_t i, k, len;

  if (y1 == y2) { 
    GLCD_DrawPixel (x1 + 1U, y2);
  }
  else {
    k = (y2 < y1) ? (-1) : (1);
    len = (int32_t)(y2 - y1);
    if (len < 0) len *= -1;

    for (i = 0; i < len; i++) {
      GLCD_DrawPixel (x1, (uint32_t)((int32_t)y1 + (k * i)));
      if (i == len / 2) x1 += 1;
    }
  }
}


/*-----------------------------------------------------------------------------
  Draw temperature graph (without scaling)
 *----------------------------------------------------------------------------*/
static void GraphDraw (void) {
  uint32_t c, cnt, x, y1, y2;
  uint16_t a, addr, sz;
  uint16_t temp, count;

  /* Determine if enough samples stored in memory */
  if (Log.WrCount < GRAPH_SAMPLES) {
    cnt = Log.WrCount;
  }
  else cnt = GRAPH_SAMPLES;

  sz = Log.MemSz;

  addr = Log.WrAddr;
  if (addr < (4U * cnt)) {
    c = cnt - Log.WrAddr/4U;
    addr = sz - ((c * 4) & 0xFFFFU);
  }
  else {
    addr -= (4U * cnt);
  }

  /* Clear graph drawing area if there are more samples that can be drawn and
     older samples fall of the graph (to clear use bargraph with value 0 to
     draw filled rectangle over the inside of graph)                          */
  if (cnt == GRAPH_SAMPLES) {
    GLCD_DrawBargraph (GRAPH_X_ORIG+1U, GRAPH_Y_ORIG-AXIS_Y_LEN+1U, AXIS_X_LEN-1U, AXIS_Y_LEN-1U, 0U);
  }

  /* Redraw graph */
  for (a = addr, c = 0U; c < cnt; c++, a += 4U) {
    if (a == sz) a = 0U;

    x = GRAPH_X_ORIG + 1U + c;
   
    if ((c + 1U) < GRAPH_SAMPLES) {
      if (Sample[c] && (Sample[c + 1])) {
        y1 = TempToY (Sample[c]);
        y2 = TempToY (Sample[c + 1]);
        ConnectSample (x, y1, y2);
      }
    }
    
    /* Read and draw current sample */
err:
    if (LOG_Read (a, &temp, &count) == 0U) {
      /* Draw current sample */
      GLCD_SetForegroundColor (GLCD_COLOR_GREEN);
      Sample[c] = temp;

      if (c > 0U) {
        /* Connect samples */
        y1 = TempToY (Sample[c-1]);
        y2 = TempToY (Sample[c  ]);
        ConnectSample (x-1U, y1, y2);
      }
      else {
        GLCD_DrawPixel (x, TempToY(Sample[c]));
      }
    }
    else goto err;
  }
}


/*-----------------------------------------------------------------------------
  Switch between modes
 *----------------------------------------------------------------------------*/
static uint32_t SwitchMode (uint32_t mode) {
  uint32_t k, n, x_px, y_px;
  uint8_t  axis_val[8];

  GLCD_SetBackgroundColor     (GLCD_COLOR_WHITE);
  GLCD_ClearScreen            ();

  switch (mode) {
    case M_INIT:
      /* Prepare display for ADC, Buttons, Joystick */
      GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
      GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
      GLCD_SetFont            (&GLCD_Font_16x24);
      GLCD_DrawString         (0U, 0U*24U, "       MCB1800      ");
      GLCD_DrawString         (0U, 1U*24U, "    Demo Example    ");
      GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
      GLCD_SetForegroundColor (GLCD_COLOR_BLUE);
      GLCD_DrawString         (0U, 2U*24U, "                    ");
      GLCD_DrawString         (0U, 3U*24U, "AD value:           ");
      GLCD_DrawString         (0U, 4U*24U, "Buttons :           ");
      GLCD_DrawString         (0U, 5U*24U, "                    ");
      GLCD_DrawString         (0U, 6U*24U, "Joystick:           ");
      GLCD_DrawString         (0U, 7U*24U, "                    ");
      GLCD_DrawString         (0U, 8U*24U, "                    ");
      GLCD_DrawString         (0U, 9U*24U, "                    ");
#ifdef _EVAL_VERSION_     // use a text button instead of bitmap with MDK-Lite in order to meet the code size limit
      GLCD_SetForegroundColor (GLCD_COLOR_GREEN);
      GLCD_DrawString         (0U, 7U*24U, "\x92\x91\x91\x91\x91\x91\x91\x91\x93"); // characters 0x90-0x95 for button frame
      GLCD_DrawString         (0U, 8U*24U, "\x90 Touch \x90");
      GLCD_DrawString         (0U, 9U*24U, "\x95\x91\x91\x91\x91\x91\x91\x91\x94");
#else
      GLCD_DrawBitmap         (BTN_X, BTN_Y, 120U, 48U, Button_16bpp);
      GLCD_DrawBitmap         (BLP_X, BLP_Y,  26U, 40U, Bulb_16bpp);
#endif
      mode = M_SCR1;
      break;

    case M_SCR1:
      /* Prepare display for thermometer and EEPROM */
      GLCD_SetBackgroundColor (GLCD_COLOR_NAVY);
      GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
      GLCD_ClearScreen        ();
      GLCD_SetFont            (&GLCD_Font_16x24);
      GLCD_DrawString         (0U, 0U*24U, "       MCB1800      ");
      GLCD_DrawString         (0U, 1U*24U, " Temperature Logger ");
      GLCD_DrawString         (0U, 2U*24U, "                    ");
      GLCD_DrawString         (0U, 3U*24U, "PCB @               ");
      GLCD_SetFont            (&GLCD_Font_6x8);

      GLCD_DrawHLine          (GRAPH_X_ORIG, GRAPH_Y_ORIG, AXIS_X_LEN);
      GLCD_DrawVLine          (GRAPH_X_ORIG, GRAPH_Y_ORIG-AXIS_Y_LEN, AXIS_Y_LEN);

      y_px = GRAPH_Y_ORIG;                          /* 20 Degrees Celsius */
      GLCD_DrawString         (GRAPH_X_ORIG-18U, y_px-4U, "20");

      y_px = GRAPH_Y_ORIG - AXIS_Y_LEN/4;           /* 25 Degrees Celsius */
      GLCD_DrawHLine          (GRAPH_X_ORIG-3U,  y_px,   4U);
      GLCD_DrawString         (GRAPH_X_ORIG-18U, y_px-4U, "25");

      y_px = GRAPH_Y_ORIG - AXIS_Y_LEN/2;           /* 30 Degrees Celsius */
      GLCD_DrawHLine          (GRAPH_X_ORIG-3U,  y_px,   4U);
      GLCD_DrawString         (GRAPH_X_ORIG-18U, y_px-4, "30");

      y_px = GRAPH_Y_ORIG - AXIS_Y_LEN*3/4;         /* 35 Degrees Celsius */
      GLCD_DrawHLine          (GRAPH_X_ORIG-3U,  y_px,   4U);
      GLCD_DrawString         (GRAPH_X_ORIG-18U, y_px-4U, "35");

      y_px = GRAPH_Y_ORIG - AXIS_Y_LEN;             /* 40 Degrees Celsius */
      GLCD_DrawHLine          (GRAPH_X_ORIG-3U,  y_px,   4U);
      GLCD_DrawString         (GRAPH_X_ORIG-18U, y_px-4, "40");
      
      k = 1;
      n = AXIS_X_LEN / 5;
      for (x_px = GRAPH_X_ORIG+n; x_px < GRAPH_X_ORIG+AXIS_X_LEN; x_px += n) {
        GLCD_DrawVLine        (x_px, GRAPH_Y_ORIG,      4U);
        sprintf ((char *)axis_val, "x+%d", n * k++);
        GLCD_DrawString       (x_px-12U, GRAPH_Y_ORIG+6U, (char *)axis_val);
      }
      GLCD_DrawString         (x_px-24U,        GRAPH_Y_ORIG+6U, "Samples");
      GLCD_DrawString         (GRAPH_X_ORIG-3U, GRAPH_Y_ORIG+6U, "x");

      mode = M_SCR2;
      break;

    case M_SCR2:
      mode = M_INIT;
      break;
    default:
      mode = M_INIT;
      break;
  }
  return (mode);
}


/*-----------------------------------------------------------------------------
  Blink LEDs
 *----------------------------------------------------------------------------*/
static void BlinkLed (void) {
  static uint32_t LED = 0U;
  static uint32_t LEDDir = 1U;

  if (LEDOn) {
    LEDOn = 0U;
    LED_On (LED);                         /* Turn specified LED on            */
  }

  if (LEDOff) {
    LEDOff = 0U;
    LED_Off (LED);                        /* Turn specified LED off           */

    if (LEDDir == 1U) {
      LED++;                              /* Change LED number                */
      if (LED == LED_GetCount () - 1U) {
        LEDDir = 0U;                      /* Change direction to down         */
      }
    }
    else {
      LED--;                              /* Change LED number                */
      if (LED == 0U) {
        LEDDir = 1U;                      /* Change direction to up           */
      }
    }
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main'
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *arg) {
  int32_t     therm;
  TOUCH_STATE tsc_state;
  uint32_t    mode;
  uint32_t    btnMsk, joyMsk;
  int32_t     adcVal;
  int32_t     tsc  = -1;
  int32_t     adc  = -1;
  uint32_t    btn  = 0xFFFFFFFFU;
  uint32_t    joy  = 0xFFFFFFFFU;

  (void) arg;

  TimerId = osTimerNew (Timer_Callback, osTimerPeriodic, (void *)0, NULL);

  if (TimerId) {
    osTimerStart (TimerId, 10U);
  }

  LOG_Init                ();           /* Logger Init                        */

  GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0U, 0U*24U, "       MCB1800      ");
  GLCD_DrawString         (0U, 1U*24U, "    Demo Example    ");
  GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor (GLCD_COLOR_BLUE);
  GLCD_DrawString         (0U, 2U*24U, "                    ");
  GLCD_DrawString         (0U, 3U*24U, " Use WAKEUP btn to  ");
  GLCD_DrawString         (0U, 4U*24U, "   switch example   ");
  GLCD_DrawString         (0U, 5U*24U, "        mode        ");
  GLCD_DrawString         (0U, 6U*24U, "                    ");
  GLCD_DrawString         (0U, 7U*24U, "                    ");
  GLCD_DrawString         (0U, 8U*24U, "  Press any key to  ");
  GLCD_DrawString         (0U, 9U*24U, "      continue      ");

  /* Wait for user to press a button */
  while (!Buttons_GetState() && !Joystick_GetState());
  mode = SwitchMode (M_INIT);

  BtnTick = 0;
  while (1) {
    BlinkLed();

    if (BtnTick && (Buttons_GetState() & BTN_WAKEUP)) {
      mode = SwitchMode (mode);
      BtnTick = 0;
      /* Force refresh */
      tsc = -1;
      adc = -1;
      btn = 0xFFFFFFFFU;
      joy = 0xFFFFFFFFU;
    }

    switch (mode) {

      /* Show potentiometer, buttons, joystick */
      case M_SCR1:
        btnMsk = Buttons_GetState();    /* Show buttons state                 */
        if (btn ^ btnMsk) {
          btn = btnMsk;
          GLCD_SetForegroundColor   (GLCD_COLOR_BLACK);
          GLCD_SetFont              (&GLCD_Font_16x24);
          if (btnMsk & BTN_P4_0)    { GLCD_DrawString ( 9U*16U, 4U*24U, "P4_0"); }
          if (btnMsk & BTN_P4_3)    { GLCD_DrawString (14U*16U, 4U*24U, "P4_3"); }
          if (btnMsk & BTN_P4_4)    { GLCD_DrawString ( 9U*16U, 5U*24U, "P4_4"); }

          GLCD_SetForegroundColor (GLCD_COLOR_LIGHT_GREY);
          if (!(btnMsk & BTN_P4_0)) { GLCD_DrawString ( 9U*16U, 4U*24U, "P4_0"); }
          if (!(btnMsk & BTN_P4_3)) { GLCD_DrawString (14U*16U, 4U*24U, "P4_3"); }
          if (!(btnMsk & BTN_P4_4)) { GLCD_DrawString ( 9U*16U, 5U*24U, "P4_4"); }
        }

        joyMsk = Joystick_GetState();   /* Show joystick state                */
        if (joy ^ joyMsk) {
          joy = joyMsk;

          if (joy == 0U) {
            GLCD_SetForegroundColor (GLCD_COLOR_LIGHT_GREY);
            GLCD_DrawString (9U*16U, 6U*24U, "NOT PUSHED");
          }
          else {
            GLCD_SetForegroundColor (GLCD_COLOR_BLACK);

            if (joy & JOYSTICK_DOWN)   { GLCD_DrawString (9U*16U, 6U*24U, "DOWN      "); }
            if (joy & JOYSTICK_UP)     { GLCD_DrawString (9U*16U, 6U*24U, "UP        "); }
            if (joy & JOYSTICK_CENTER) { GLCD_DrawString (9U*16U, 6U*24U, "CENTER    "); }
            if (joy & JOYSTICK_LEFT)   { GLCD_DrawString (9U*16U, 6U*24U, "LEFT      "); }
            if (joy & JOYSTICK_RIGHT)  { GLCD_DrawString (9U*16U, 6U*24U, "RIGHT     "); }
          }
        }
        
        ADC_StartConversion();          /* Show A/D conversion bargraph       */
        adcVal = ADC_GetValue();
        if ((adcVal != -1) && (adc ^ adcVal)) {
          adc = adcVal;
          GLCD_SetForegroundColor (GLCD_COLOR_GREEN);
          GLCD_DrawBargraph       (144U, 3U*24U, 160U, 20U, ((uint32_t)adcVal * 100U) / ((1U << ADC_GetResolution()) - 1U));
        }
        
        if (Touch_GetState (&tsc_state) == 0U) {   /* Get touch screen state   */
          if (tsc ^ tsc_state.pressed) {
            tsc = tsc_state.pressed;
            if ((tsc_state.pressed) &&
               (tsc_state.x >= 200 && tsc_state.x <= 1000 &&
                tsc_state.y >= 260 && tsc_state.y <= 1650)) {
#ifdef _EVAL_VERSION_     // use a text button instead of bitmap with MDK-Lite in order to meet the code size limit
              GLCD_SetForegroundColor(GLCD_COLOR_DARK_GREEN);
              GLCD_SetFont    (&GLCD_Font_16x24);
              GLCD_DrawString (0U, 7U*24U, "\x92\x91\x91\x91\x91\x91\x91\x91\x93"); // characters 0x90-0x95 for button frame
              GLCD_DrawString (0U, 8U*24U, "\x90 Touch \x90");
              GLCD_DrawString (0U, 9U*24U, "\x95\x91\x91\x91\x91\x91\x91\x91\x94");
#else
              GLCD_DrawBitmap (BTN_X, BTN_Y, 120U, 48U, &Button_16bpp[1*11520]);
              GLCD_DrawBitmap (BLP_X, BLP_Y,  26U, 40U, &Bulb_16bpp[1*2080]);
#endif
            }
            else {
#ifdef _EVAL_VERSION_     // use a text button instead of bitmap with MDK-Lite in order to meet the code size limit
              GLCD_SetForegroundColor(GLCD_COLOR_GREEN);
              GLCD_SetFont    (&GLCD_Font_16x24);
              GLCD_DrawString (0U, 7U*24U, "\x92\x91\x91\x91\x91\x91\x91\x91\x93"); // characters 0x90-0x95 for button frame
              GLCD_DrawString (0U, 8U*24U, "\x90 Touch \x90");
              GLCD_DrawString (0U, 9U*24U, "\x95\x91\x91\x91\x91\x91\x91\x91\x94");
#else
              GLCD_DrawBitmap (BTN_X, BTN_Y, 120U, 48U, &Button_16bpp[0*11520]);
              GLCD_DrawBitmap (BLP_X, BLP_Y,  26U, 40U, &Bulb_16bpp[0*2080]);
#endif
            }
          }
        }
        break;

      case M_SCR2:
        /* Show temperature and EEPROM graph */
        GLCD_SetForegroundColor   (GLCD_COLOR_GREEN);

        if (Measure) {
          Measure = 0;
          /* Get temperature */
          therm = Thermometer_GetTemperature ();
          if (therm != TEMPERATURE_INVALID) {
            sprintf (StringBuf, "%.1f C", ((double)therm) / 10.0);
            GLCD_SetFont    (&GLCD_Font_16x24);
            GLCD_DrawString (10U*16U, 3U*24U, (char *)StringBuf);
            LOG_Write ((uint16_t)therm);
          }
          /* Draw temperature graph */
          GraphDraw();
        }
        break;

      default:
        mode = SwitchMode (mode);
        break;
    }
  }
}
